#!/usr/bin/perl

# Converts a D or ML source file to TeX.  Understands the following
# markup language (implemented as D/ML comments so that the source can
# still be run).  
#
#   (*@omitblock*)..(*@endblock*)
# This everything between these two tags will be omitted from the .tex
# version.  The tags must be on lines by themselves.
#
#   (*@add 'text'*)
# 'text' (no quotes) will be placed in the output verbatim.  This tag
# may not span lines.
#

use strict;

my $omitBlock = 0;

while (<>) {
  if ($omitBlock) {
    /\(\*\@endblock\*\)/ and $omitBlock = 0;
  }
  elsif (/\(\*\@omitblock\*\)/) {
    $omitBlock = 1;
  }
  else {
    # Escape characters that are special in TeX.
    s/\\/\\verb|\\|/g;
    s/\{/\\\{/g;
    s/\}/\\\}/g;
    s/\_/\\\_/g;
    
    s/\(\*\@add\s+\'([^\']*)\'\s*\*\)/\1/g;
    print;
  }
}
